SystemOrganization addCategory: #TinyBlog!
SystemOrganization addCategory: 'TinyBlog-Tests'!

TestCase subclass: #TBMBlogTest
	instanceVariableNames: 'blog numberOfPosts additionalEmptyPost category1 category2 numberOfPostsFromCategory1 numberOfPostsFromCategory2 numberOfPostsUnclassified numberOfCategories categoriesClass numberOfCategoriesFromVisiblePosts numberOfVisiblePosts numberOfVisiblePostsUnclassified'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TinyBlog-Tests'!

!TBMBlogTest methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 5/11/2016 11:48'!
setUp

	TBMBlog current removeAllBlogPosts.
	blog := TBMBlog createDemoPosts current.
	
	numberOfPosts := 5.
	numberOfVisiblePosts := 5.
	numberOfVisiblePostsUnclassified := 1.
	
	category1 := 'TinyBlog'.
	category2 := 'Pharo'.
	
	categoriesClass := TBMPost unclassifiedTag class.
	
	numberOfPostsFromCategory1 := 1.
	numberOfPostsFromCategory2 := 3.
	numberOfPostsUnclassified  := 1.
	
	numberOfCategories := 3.
	numberOfCategoriesFromVisiblePosts := 3.

	
	additionalEmptyPost := TBMPost new! !

!TBMBlogTest methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 5/10/2016 10:38'!
tearDown
	"reset the current blog at the end of the tests"

	TBMBlog reset! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/11/2016 11:40'!
testAddBlogPost

	blog addBlogPost: additionalEmptyPost.
	self assert: blog size equals: numberOfPosts + 1! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/11/2016 11:12'!
testAllBlogPosts

	| allPosts |
	
	allPosts := blog allBlogPosts.
	self assert: allPosts size equals: numberOfPosts.
	allPosts do: [ :eachPost | 
		self assert: eachPost class = TBMPost]
! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/11/2016 11:16'!
testAllBlogPostsFromCategory

	| postsFromCategory1 postsFromCategory2 postsUnclassified |

	postsFromCategory1 := blog allBlogPostsFromCategory: category1.
	self assert: postsFromCategory1 size = numberOfPostsFromCategory1.
	postsFromCategory1 do: [ :eachPost | self assert: eachPost class = TBMPost ].
	
	postsFromCategory2 := blog allBlogPostsFromCategory: category2.
	self assert: postsFromCategory2 size = numberOfPostsFromCategory2.
	postsFromCategory2 do: [ :eachPost | self assert: eachPost class = TBMPost ].
	
	postsUnclassified := blog allBlogPostsFromCategory: TBMPost unclassifiedTag.
	self assert: postsUnclassified size = numberOfPostsUnclassified.
	postsUnclassified do: [ :eachPost | self assert: eachPost class = TBMPost ].
	
	! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/11/2016 11:22'!
testAllCategories

	| allCategories |

	allCategories := blog allCategories.
	self assert: allCategories size = numberOfCategories.
	allCategories do: [ :eachCategory | 
		self assert: eachCategory class = categoriesClass ].
	
! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/11/2016 11:26'!
testAllCategoriesFromVisibleBlogPosts

	| allCategoriesFromVisiblePosts |

	allCategoriesFromVisiblePosts := blog allCategoriesFromVisibleBlogPosts.
	self assert: allCategoriesFromVisiblePosts size = numberOfCategoriesFromVisiblePosts.
	allCategoriesFromVisiblePosts do: [ :eachCategory | 
		self assert: eachCategory class = categoriesClass ].! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/11/2016 11:41'!
testAllNotVisibleBlogPosts

	| allNotVisiblePosts |
	
	blog addBlogPost: additionalEmptyPost.
	allNotVisiblePosts := blog allNotVisibleBlogPosts.
	self assert: allNotVisiblePosts size = (numberOfPosts - numberOfVisiblePosts + 1).
	allNotVisiblePosts do: [ :eachPost | 
		self assert: eachPost class = TBMPost]
	! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/11/2016 11:42'!
testAllVisibleBlogPosts
	| allVisiblePosts |
	
	blog addBlogPost: additionalEmptyPost.
	allVisiblePosts := blog allVisibleBlogPosts.
	self assert: allVisiblePosts size = numberOfVisiblePosts.
	allVisiblePosts do: [ :eachPost | 
		self assert: eachPost class = TBMPost]! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/11/2016 12:22'!
testAllVisibleBlogPostsFromCategory
	| allVisiblePostsUnclassified |
	
	blog addBlogPost: additionalEmptyPost.
	allVisiblePostsUnclassified := blog allVisibleBlogPostsFromCategory: TBMPost unclassifiedTag.
	self assert: allVisiblePostsUnclassified size = numberOfVisiblePostsUnclassified.
	allVisiblePostsUnclassified do: [ :eachPost | self assert: eachPost class = TBMPost]! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/11/2016 12:22'!
testCategoryBeNotVisible

	| postsUnclassified visiblePostsUnclassified |
	
	postsUnclassified := blog categoryBeNotVisible: TBMPost unclassifiedTag.
	self assert: postsUnclassified size = numberOfPostsUnclassified.
	postsUnclassified do: [ :eachPost | self assert: eachPost class = TBMPost ].
	visiblePostsUnclassified := blog allVisibleBlogPostsFromCategory: TBMPost unclassifiedTag.
	self assert: visiblePostsUnclassified size = 0.
! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/11/2016 12:26'!
testCategoryBeVisible
	| postsUnclassified visiblePostsUnclassified |
	
	blog addBlogPost: additionalEmptyPost.
	postsUnclassified := blog categoryBeVisible: TBMPost unclassifiedTag.
	self assert: postsUnclassified size = (numberOfPostsUnclassified + 1).
	postsUnclassified do: [ :eachPost | self assert: eachPost class = TBMPost ].
	visiblePostsUnclassified := blog allVisibleBlogPostsFromCategory: TBMPost unclassifiedTag.
	self assert: visiblePostsUnclassified size = (numberOfVisiblePostsUnclassified + 1).! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/11/2016 12:33'!
testCategoryRenameFromTo
	
	| postsFromCategory1 postsUnclassified |

	postsUnclassified := blog categoryRenameFrom: TBMPost unclassifiedTag  to: category1.
	self assert: postsUnclassified size = numberOfPostsUnclassified.
	postsUnclassified do: [ :eachPost |  self assert: eachPost class = TBMPost ].
	postsFromCategory1 := blog allBlogPostsFromCategory: category1.
	self assert: postsFromCategory1 size = (numberOfPostsFromCategory1 + numberOfPostsUnclassified)
! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/11/2016 12:34'!
testRemoveAllBlogPosts

	blog removeAllBlogPosts.
	self assert: blog size equals: 0! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/11/2016 12:39'!
testRemoveAllBlogPostsFromCategory


	| postsFromCategory2 |
	
	blog removeAllBlogPostsFromCategory: category2.
	postsFromCategory2 := blog allBlogPostsFromCategory: category2.
	self assert: postsFromCategory2 size = 0.
! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/11/2016 10:36'!
testSize
	"verify the blog size after tests setup"

	self assert: blog size equals: numberOfPosts ! !

Object subclass: #TBMBlog
	instanceVariableNames: 'posts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TinyBlog'!

TBMBlog class
	instanceVariableNames: 'uniqueInstance'!

!TBMBlog commentStamp: 'FrancoisJarriges 5/10/2016 15:46' prior: 0!
I represent a unique blog  in the Tiny Blog (FUN MOOC - Programmation immersive en Pharo)

I provide methods on  the class side to manage the unique instance of myself 

I provide methods to record posts to myself and manage them through the criterias defined in their class

Public API and Key Messages :

aPost := TBMPost title: 'A title'  text: 'A text'  category: 'A category' .

blog :=  TBMBlog current.		"unique instance of TBMBlog"
blog writeBlogPost:  aPost.	"add a post to the blog"

"Get selected posts in an OrderedCollection of posts :"

allPosts :=  blog allBlogPosts.
allPostsFromACategory :=	blog allBlogPostsFromCategory:  'A category'.
allVisiblePosts := blog allVisibleBlogPosts.
allVisiblePostsFromACategory := blog allVisibleBlogPostsFromCategory:  'A category'.

"Get  an actual Set  of  all categories among the selected posts :"

allCategories := blog allCategories.
allCategoriesFromVisiblePosts := blog allCategoriesFromVisibleBlogPosts.

"Get the actual number of posts :"
numberOfPosts := blog size.

"Clear the selected posts from the blog :"
blog removeAllBlogPostsFromCategory: 'A category'.
blog removeAllPosts .

Internal Representation and Key Implementation Points.

    Instance Variables
	posts:	<OrderedCollection>
		
     Class Instance Variables 
	uniqueInstance: 	<TBMBlog>
	
    Implementation Points!

TBMBlog class
	instanceVariableNames: 'uniqueInstance'!

!TBMBlog class methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 5/11/2016 10:24'!
createDemoPosts
"TBMBlog createDemoPosts"
	
self current

	addBlogPost: ((TBMPost 
		title: 'Welcome in TinyBlog' 
		text: 'TinyBlog is a small blog engine made with Pharo.' 
		category: 'TinyBlog') 
		visible: true);
		
	addBlogPost: ((TBMPost 
		title: 'Report Pharo Sprint' 
		text: 'Friday, June 12 there was a Pharo sprint / Moose dojo. It was a nice event with more than 15 motivated sprinters. With the help of candies, cakes and chocolate, huge work has been done'
		category: 'Pharo') 
		visible: true);
		
	addBlogPost: ((TBMPost 
		title: 'Brick on top of Bloc - Preview' 
		text: 'We are happy to announce the first preview version of Brick, a new widget set created from scratch on top of Bloc. Brick is being developed primarily by Alex Syrel (together with Alain Plantec, Andrei Chis and myself), and the work is sponsored by ESUG. Brick is part of the Glamorous Toolkit effort and will provide the basis for the new versions of the development tools.'
		category: 'Pharo') 
		visible: true);
		
	addBlogPost: ((TBMPost 
		title: 'The sad story of unclassified blog posts'
		text: 'So sad that I can read this.') 
		visible: true);
		
	addBlogPost: ((TBMPost 
		title: 'Working with Pharo on the Raspberry Pi' 
		text: 'Hardware is getting cheaper and many new small devices like the famous Raspberry Pi provide new computation power that was one once only available on regular desktop computers.' 
		category: 'Pharo') 
		visible: true)! !

!TBMBlog class methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 5/10/2016 13:26'!
current
	"answer the unique instance of the TBMBlog and create it if it doesn't exist yet"
	
	^ uniqueInstance ifNil: [ uniqueInstance := self new ]! !

!TBMBlog class methodsFor: 'initializing' stamp: 'FrancoisJarriges 5/10/2016 13:24'!
initialize
	"reset the TBMBlog instance singleton when TBMBlog class is loaded "

	self reset ! !

TBMBlog initialize!

!TBMBlog class methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 5/10/2016 10:06'!
reset
	uniqueInstance := nil! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 17:53'!
addBlogPost: aTBMPost
	"write the given post in the current blog database"

	posts add: aTBMPost! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 11:26'!
allBlogPosts
	"answer the entire posts collection defined in the current blog"

	^ posts! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 15:33'!
allBlogPostsFromCategory: aString 
	"answer a selection of all the posts classified in the given category from the current blog"
	
	^ posts select: [ :eachPost | eachPost category = aString ]! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 13:11'!
allCategories
	"answer a set of all the posts categories defined in the current blog"
	
	^ (posts collect: [ :eachPost | eachPost category ]) asSet! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 15:34'!
allCategoriesFromVisibleBlogPosts
	"answer a set of all the categories defined among visible posts from the current blog"
	
	^ ((posts select: [ :eachPost | eachPost isVisible ]) 
		collect: [ :eachVisiblePost | eachVisiblePost category ]) asSet! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 18:01'!
allNotVisibleBlogPosts
	"answer a selection of all the unvisible posts from the current blog"
	
	^ posts select: [ :eachPost | eachPost isVisible not]! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 15:34'!
allVisibleBlogPosts
	"answer a selection of all the visible posts from the current blog"
	
	^ posts select: [ :eachPost | eachPost isVisible ]! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/11/2016 12:28'!
allVisibleBlogPostsFromCategory: aString 
	"answer a selection of all the visible posts in the given category from the current blog"
	
	^ posts select: [ :eachPost | (eachPost category = aString) & (eachPost isVisible) ]! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/11/2016 12:28'!
categoryBeNotVisible: aString 
	"make each post unvisible within the given category identified as aString
	answer a selection of all the posts classified in the given category from the current blog"
	
	| postsFromSelectedCategory |
	
	postsFromSelectedCategory := self allBlogPostsFromCategory: aString.
	postsFromSelectedCategory do: [ :eachPostFromSelectedCategory | eachPostFromSelectedCategory beNotVisible ].
	^ postsFromSelectedCategory! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/11/2016 12:29'!
categoryBeVisible: aString 
	"make each post visible within the given category identified as aString
	answer a selection of all the posts classified in the given category from the current blog"
	
	| postsFromSelectedCategory |
	
	postsFromSelectedCategory := self allBlogPostsFromCategory: aString.
	postsFromSelectedCategory do: [ :eachPostFromSelectedCategory | eachPostFromSelectedCategory beVisible ].
	^ postsFromSelectedCategory! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/11/2016 12:29'!
categoryRenameFrom: actualString to: newString
	"rename the actual category identified as actualString by giving it newString as new name
	answer a selection of all the posts classified in the given category from the current blog"

	| postsFromSelectedCategory |
	
	postsFromSelectedCategory := self allBlogPostsFromCategory: actualString.
	postsFromSelectedCategory do: [ :eachPostFromSelectedCategory | 
		eachPostFromSelectedCategory category: newString ].
	^ postsFromSelectedCategory! !

!TBMBlog methodsFor: 'initializing' stamp: 'FrancoisJarriges 5/10/2016 10:04'!
initialize

	super initialize.
	posts := OrderedCollection new! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 15:12'!
removeAllBlogPosts
	"remove all posts by creating a new empty collection for the current blog"

	posts := OrderedCollection new! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 15:45'!
removeAllBlogPostsFromCategory: aString
	"remove all the posts classified in the given category from the current blog"

	posts	removeAllSuchThat: [ :eachPost | eachPost category = aString ]! !

!TBMBlog methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 5/10/2016 10:57'!
size
	"answer the size of the posts collection in the current blog"

	^ posts size! !

Object subclass: #TBMPost
	instanceVariableNames: 'title text date category visible'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TinyBlog'!

!TBMPost commentStamp: 'FrancoisJarriges 5/10/2016 14:09' prior: 0!
I represent a post in the Tiny Blog (FUN MOOC - Programmation immersive en Pharo)

I provide methods to create and manage  a post


Public API and Key Messages :

post1 := TBMPost title: 'test post 1'	text: 'created unclassified and unvisible' .
post2 := TBMPost title: 'test post 2'  text: 'created unvisible'  category: 'Testing' .

post1 isUnclassified ifTrue:  [post1 category: 'Testing'] .
post1 isVisible  ifFalse:  [post1 beVisible ] .

post1 beNotVisible; category: self class unclassifiedTag .



Internal Representation and Key Implementation Points.

    Instance Variables
	category:	<String>
	date:		<Date>
	text:		<Text>
	title:		<String>
	visible:		<Boolean>


    Implementation Points!

!TBMPost class methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 5/5/2016 10:47'!
title: aTitle text: aText

	^ self new 
		title: aTitle;
		text: aText;
		yourself ! !

!TBMPost class methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 5/5/2016 10:49'!
title: aTitle text: aText category: aCategory

	^ (self title: aTitle text: aText) 
		category: aCategory;
		yourself ! !

!TBMPost class methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 5/4/2016 17:08'!
unclassifiedTag

	^ 'Unclassified'! !

!TBMPost methodsFor: 'visibility' stamp: 'FrancoisJarriges 5/4/2016 17:04'!
beNotVisible

	self visible: false! !

!TBMPost methodsFor: 'visibility' stamp: 'FrancoisJarriges 5/4/2016 17:03'!
beVisible

	self visible: true! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/4/2016 17:01'!
category

	^ category! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/9/2016 08:36'!
category: aString

	category := aString! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/4/2016 17:00'!
date

	^ date! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/9/2016 08:54'!
date: aDate

	date := aDate! !

!TBMPost methodsFor: 'initialization' stamp: 'FrancoisJarriges 5/10/2016 14:11'!
initialize

	super initialize.
	self category: self class unclassifiedTag.
	self date: Date today.
	self beNotVisible ! !

!TBMPost methodsFor: 'testing' stamp: 'FrancoisJarriges 5/5/2016 10:57'!
isUnclassified

	^ self category = self class unclassifiedTag ! !

!TBMPost methodsFor: 'testing' stamp: 'FrancoisJarriges 5/4/2016 17:04'!
isVisible

	^ self visible! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/4/2016 16:59'!
text

	^ text! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/9/2016 08:20'!
text: aText

	text := aText ! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/4/2016 16:58'!
title

	^ title! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/9/2016 08:31'!
title: aString

	title := aString ! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/4/2016 17:01'!
visible

	^ visible! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/5/2016 10:34'!
visible: aBoolean

	visible := aBoolean  ! !

Object subclass: #TBPost
	instanceVariableNames: 'category date text title visible'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TinyBlog'!

!TBPost commentStamp: 'LucFabresse 3/18/2016 10:01' prior: 0!
I represent blog posts.
Inspect the code below :

TBPost
	title: 'Welcome in TinyBlog'
	text: 'TinyBlog is a small blog engine made with Pharo.' 
	category: 'TinyBlog'

!

!TBPost class methodsFor: 'instance creation' stamp: 'StephaneDucasse 12/4/2015 14:44'!
title: aTitle text: aText 
	^ self new
		title: aTitle;
		text: aText;
		yourself! !

!TBPost class methodsFor: 'instance creation' stamp: 'StephaneDucasse 12/4/2015 14:45'!
title: aTitle text: aText category: aCategory
	^ (self title: aTitle text: aText)
			category: aCategory;
			yourself! !

!TBPost class methodsFor: 'constants' stamp: 'StephaneDucasse 12/4/2015 14:36'!
unclassifiedTag
	^ 'Unclassified'! !

!TBPost methodsFor: 'visibility' stamp: 'StephaneDucasse 12/4/2015 14:35'!
beVisible
   self visible: true! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:32'!
category
	^ category! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:33'!
category: anObject
	category := anObject
! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:33'!
date
	^ date! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:33'!
date: anObject
	date := anObject! !

!TBPost methodsFor: 'initialize' stamp: 'StephaneDucasse 12/4/2015 14:39'!
initialize
	self category: self class unclassifiedTag.
	self date: Date today.
	self notVisible! !

!TBPost methodsFor: 'testing' stamp: 'StephaneDucasse 12/4/2015 14:48'!
isUnclassified
   ^ self category = self class unclassifiedTag! !

!TBPost methodsFor: 'testing' stamp: 'StephaneDucasse 12/4/2015 14:35'!
isVisible
   ^ self visible! !

!TBPost methodsFor: 'visibility' stamp: 'StephaneDucasse 12/4/2015 14:35'!
notVisible
   self visible: false! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:33'!
text
	^ text! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:33'!
text: anObject
	text := anObject! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:33'!
title
	^ title! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:33'!
title: anObject
	title := anObject! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:33'!
visible
	^ visible! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:33'!
visible: anObject
	visible := anObject! !

TBMBlog initialize!
